package com.ims.common.utils;


import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

public class DateUtils {

	private static final String format = "yyyy-MM-dd";
	private static final String format1 = "yyyy-MM-dd HH:mm:ss";
	private static final String format2 = "MM/dd/yyyy HH:mm:ss";
	private static final String format3 = "yyyy-MM";

	// 第一次调用get将返回null
	private static ThreadLocal<SimpleDateFormat> threadLocal = new ThreadLocal<SimpleDateFormat>();

	// 获取线程的变量副本，如果不覆盖initialValue，第一次get返回null，故需要初始化一个SimpleDateFormat，并set到threadLocal中

	/**
	 * yyyy-MM-dd
	 *
	 * @return
	 */
	public static SimpleDateFormat getFormat() {
		SimpleDateFormat df = (SimpleDateFormat) threadLocal.get();
		if (df == null) {
			df = new SimpleDateFormat(format);
			threadLocal.set(df);
		}
		return df;

	}


	/**
	 * yyyy-MM-dd HH:mm:ss
	 *
	 * @return
	 */
	public static SimpleDateFormat getFormat1() {

		SimpleDateFormat df1 = (SimpleDateFormat) threadLocal.get();

		if (df1 == null) {
			df1 = new SimpleDateFormat(format1);
			threadLocal.set(df1);
		}

		return df1;

	}

	/**
	 * MM/dd/yyyy HH:mm:ss
	 *
	 * @return
	 */
	public static SimpleDateFormat getFormat2() {

		SimpleDateFormat df2 = (SimpleDateFormat) threadLocal.get();

		if (df2 == null) {
			df2 = new SimpleDateFormat(format2);
			threadLocal.set(df2);
		}

		return df2;

	}

	/**
	 * yyyy-MM
	 *
	 * @return
	 */
	public static SimpleDateFormat getFormat3() {

		SimpleDateFormat df3 = (SimpleDateFormat) threadLocal.get();

		if (df3 == null) {
			df3 = new SimpleDateFormat(format3);
			threadLocal.set(df3);
		}

		return df3;

	}

	/**
	 * 获取当前日期
	 *
	 * @param
	 * @return Date
	 */
	public static Date getCurrentDate() {
		Calendar calendar = Calendar.getInstance();
		Date date = calendar.getTime();
		return date;
	}


	public static String toDate(Date date) {
		return getFormat().format(date);
	}

	public static Date toDate(String date) {
		try {
			return getFormat().parse(date);
		} catch (ParseException e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * 获取年份
	 *
	 * @param date
	 * @return
	 */
	public static int getYear(Date date) {
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(date);
		int year = calendar.get(Calendar.YEAR);
		return year;
	}

	/**
	 * 获取月份
	 *
	 * @param date
	 * @return
	 */
	public static int getMonth(Date date) {
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(date);
		int month = calendar.get(Calendar.MONTH)+1;
		return month;
	}

	/**
	 * 获取当前时间季度
	 *
	 * @param date
	 * @return
	 */
	public static int getSeason(Date date) {
		int month = 0;
		try {
			Calendar calendar = Calendar.getInstance();
			calendar.setTime(date);
			//由于month从0开始，所有这里+1
			month = calendar.get(Calendar.MONTH) + 1;
		} catch (Exception e) {
			return 0;
		}
		int quarter = month % 3 == 0 ? month / 3 : month / 3 + 1;
		return quarter;
	}


	public static String[] getSeasonDay(Date date) {
		SimpleDateFormat sdf = new SimpleDateFormat(format);
		if (null == date) {
			date = new Date();
		}
		Calendar c = Calendar.getInstance();
		c.setTime(date);
		int month = c.get(Calendar.MONTH);
		//数学原理：
		// moth属性=月份-1，3个月=1季度，
		// 那么季度的最大月份÷3 =季度，(月份-1(即month属性))÷3向下取整 = 上一个季度；
		// 上个季度*3=上个季度最大月份=当前季度最小月份-1=当前季度最小月份month属性
		Calendar startC = Calendar.getInstance();
		startC.setTime(date);
		int seasonStartMonthNum = (int) (Math.floor(month / 3.0) * 3);
		startC.set(Calendar.MONTH, seasonStartMonthNum);
		startC.set(Calendar.DAY_OF_MONTH, 1);
		String seasonStartDateStr = sdf.format(startC.getTime());
		Calendar endC = Calendar.getInstance();
		endC.setTime(date);
		//当前季度最大月=当前最小月+2；
		int seasonEndMonthNum;
		seasonEndMonthNum = seasonStartMonthNum + 2;
		endC.set(Calendar.MONTH, seasonEndMonthNum);
		endC.set(Calendar.DAY_OF_MONTH, endC.getActualMaximum(Calendar.DAY_OF_MONTH));
		String seasonEndDateStr = sdf.format(endC.getTime());
		return new String[]{seasonStartDateStr, seasonEndDateStr};
	}


}
