package com.ims.eval.controller;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.EvaluateReport;
import com.ims.eval.entity.EvaluateRule;
import com.ims.eval.entity.Indicator;
import com.ims.eval.entity.dto.result.R;
import com.ims.eval.service.IEvaluateReportService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author wang
 * @since 2023-03-25
 */
@RestController
@RequestMapping("//evaluate-report")
public class EvaluateReportController {


	@Autowired
	private IEvaluateReportService evaluateReportService;


	/**
	 * 生成考评报告
	 *
	 * @param organizationEvaluationId 组织考评主键
	 * @param backroll                 重算表示
	 * @return
	 */
	//@ImsPreAuth("eval:dataDictionary:view")
	@PostMapping(value = "generateEvaluateReport")
	public R generateEvaluateReport(@RequestParam(value = "organizationEvaluationId", required = false) String organizationEvaluationId,
									@RequestParam(value = "backroll", required = false) boolean backroll) {
		try {
			boolean b = evaluateReportService.generateEvaluateReport(organizationEvaluationId,backroll);
			return R.ok().data(b);
		} catch (CustomException e) {
			return R.customError(e.getMessage());
		}

	}


	/**
	 * 查询分页数据
	 *
	 * @param pageNum
	 * @param pageSize
	 * @param id                       主键
	 * @param evaluateReportName       报告名称
	 * @param binSection               业务版块
	 * @param organizationEvaluationId 考评记录id
	 * @param organizationType         考评类别(单位、部门)
	 * @param checkCycle               周期
	 * @param year                     年
	 * @param month                    月
	 * @return
	 */
	//@ImsPreAuth("eval:dataDictionary:view")
	@GetMapping(value = "list")
	public R list(@RequestParam(value = "pageNum") Integer pageNum,
				  @RequestParam(value = "pageSize") Integer pageSize,
				  @RequestParam(value = "id", required = false) String id,
				  @RequestParam(value = "evaluateReportName", required = false) String evaluateReportName,
				  @RequestParam(value = "binSection", required = false) String binSection,
				  @RequestParam(value = "organizationEvaluationId", required = false) String organizationEvaluationId,
				  @RequestParam(value = "organizationType", required = false) String organizationType,
				  @RequestParam(value = "checkCycle", required = false) String checkCycle,
				  @RequestParam(value = "year", required = false) String year,
				  @RequestParam(value = "month", required = false) String month) {
		IPage<EvaluateReport> list = evaluateReportService.list(pageNum, pageSize, id, evaluateReportName, binSection, organizationEvaluationId, organizationType, checkCycle, year, month);
		return R.ok().data(list);
	}


}
