package com.ims.eval.controller;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.dto.result.R;
import com.ims.eval.entity.EvaluateRule;
import com.ims.eval.service.IEvaluateRuleService;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;

/**
 * <p>
 * 考评规则表 前端控制器
 * </p>
 *
 * @author wang
 * @since 2023-02-27
 */
@RestController
@RequestMapping("//evaluate-rule")
public class EvaluateRuleController {


	@Autowired
	private IEvaluateRuleService evaluateRuleService;

	/**
	 * 查询 list
	 *
	 * @param id         主键
	 * @param des        描述
	 * @param binSection 业务版块
	 * @param binStage   业务阶段
	 * @return
	 */
	//@ImsPreAuth("eval:dataDictionary:view")
	@GetMapping(value = "list")
	public R list(@RequestParam(value = "pageNum") Integer pageNum,
				  @RequestParam(value = "pageSize") Integer pageSize,
				  @RequestParam(value = "id", required = false) String id,
				  @RequestParam(value = "des", required = false) String des,
				  @RequestParam(value = "binSection", required = false) String binSection,
				  @RequestParam(value = "binStage", required = false) String binStage) {
		IPage<EvaluateRule> list = evaluateRuleService.listPage(pageNum, pageSize, id, des, binSection, binStage);
		return R.ok().data(list);
	}


	/**
	 * 查询 listAll
	 *
	 * @param id         主键
	 * @param des        描述
	 * @param binSection 业务版块
	 * @param binStage   业务阶段
	 * @return
	 */
	//@ImsPreAuth("eval:dataDictionary:view")
	@GetMapping(value = "listAll")
	public R listAll(@RequestParam(value = "id", required = false) String id,
				  @RequestParam(value = "des", required = false) String des,
				  @RequestParam(value = "binSection", required = false) String binSection,
				  @RequestParam(value = "binStage", required = false) String binStage) {
		List<EvaluateRule> list = evaluateRuleService.listAll(id, des, binSection, binStage);
		return R.ok().data(list);
	}


	/**
	 * 添加
	 *
	 * @param evaluateRule
	 * @return
	 */

	//@ImsPreAuth("eval:dataDictionary:edit")
	@PostMapping(value = "/save")
	@ApiOperation(value = "新增(修改)", notes = "新增（修改）")
	public R addAll(@RequestBody EvaluateRule evaluateRule) {

		try {
			boolean b = evaluateRuleService.saveOrUpdate(evaluateRule);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error().data("保存失败！");
			}
		} catch (CustomException e){
			return R.customError(e.getMessage()).data("失败！");
		}
	}


	/**
	 * 批量删除
	 *
	 * @param ids
	 * @return
	 */
	//@ImsPreAuth("eval:dataDictionary:remove")
	@PostMapping(value = "/remove/{ids}")
	@ApiOperation(value = "删除", notes = "删除")
	public R deleteAll(@PathVariable("ids") String ids) {
		String[] strings = ids.split(",");
		boolean b = evaluateRuleService.removeByIds(Arrays.asList(strings));
		if (b) {
			return R.ok().data(b);
		} else {
			return R.error().data("删除失败！");
		}
	}
}
