package com.ims.eval.controller;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.dto.request.IndicatorDTO;
import com.ims.eval.entity.dto.request.IndicatorDictionaryDTO;
import com.ims.eval.entity.dto.result.R;
import com.ims.eval.entity.Indicator;
import com.ims.eval.service.IIndicatorDictionaryService;
import com.ims.eval.service.IIndicatorService;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * <p>
 * 指标 前端控制器
 * </p>
 *
 * @author wang
 * @since 2023-02-26
 */
@Slf4j
@RestController
@RequestMapping("//indicator")
public class IndicatorController {


	@Autowired
	private IIndicatorService iIndicatorService;

	@Autowired
	private IIndicatorDictionaryService indicatorDictionaryService;


	/**
	 * 查询分页数据
	 *
	 * @param id            主键ID
	 * @param indicatorName 指标名称
	 * @param indicatorCode 指标编码
	 * @param binSection    业务版块
	 * @param binStage      业务阶段
	 * @param dept          部门
	 * @param company       公司
	 * @return
	 */
	//@ImsPreAuth("eval:dataDictionary:view")
	@GetMapping(value = "list")
	public R list(@RequestParam(value = "pageNum") Integer pageNum,
				  @RequestParam(value = "pageSize") Integer pageSize,
				  @RequestParam(value = "id", required = false) String id,
				  @RequestParam(value = "indicatorName", required = false) String indicatorName,
				  @RequestParam(value = "indicatorCode", required = false) String indicatorCode,
				  @RequestParam(value = "binSection", required = false) String binSection,
				  @RequestParam(value = "binStage", required = false) String binStage,
				  @RequestParam(value = "dept", required = false) String dept,
				  @RequestParam(value = "company", required = false) String company) {
		IPage<Indicator> list = iIndicatorService.list2(pageNum, pageSize, id, indicatorName, indicatorCode, binSection, binStage, dept, company);
		return R.ok().data(list);
	}


	/**
	 * 查询所有数据
	 *
	 * @param id            主键ID
	 * @param indicatorName 指标名称
	 * @param indicatorCode 指标编码
	 * @param binSection    业务版块
	 * @param binStage      业务阶段
	 * @param dept          部门
	 * @param company       公司
	 * @return
	 */
	//@ImsPreAuth("eval:dataDictionary:view")
	@GetMapping(value = "listAll")
	public R listAll(
		@RequestParam(value = "id", required = false) String id,
		@RequestParam(value = "indicatorName", required = false) String indicatorName,
		@RequestParam(value = "indicatorCode", required = false) String indicatorCode,
		@RequestParam(value = "binSection", required = false) String binSection,
		@RequestParam(value = "binStage", required = false) String binStage,
		@RequestParam(value = "dept", required = false) String dept,
		@RequestParam(value = "company", required = false) String company) {
		List<Indicator> list = iIndicatorService.listAll(id, indicatorName, indicatorCode, binSection, binStage, dept, company);
		return R.ok().data(list);
	}

	/**
	 * 根据多指标查询指标项
	 *
	 * @param indicatorCode 指标编码
	 * @return
	 */
	@GetMapping(value = "selectByCode")
	public R selectByCode(
		@RequestParam(value = "indicatorCede", required = false) String indicatorCode) {

		List<IndicatorDictionaryDTO> list = new ArrayList<>();
		Indicator indicator = iIndicatorService.selectByCode(indicatorCode);
		if(indicator != null){
			 list = indicatorDictionaryService.list(indicator.getId(),"","");
		}
		return R.ok().data(list);
	}



	/**
	 * 根据指标id获取主子指标
	 *
	 * @param id 指标id
	 * @return
	 */
	@GetMapping(value = "selectById")
	public R selectById(
		@RequestParam(value = "id", required = false) String id) {

		List<IndicatorDictionaryDTO> list = new ArrayList<>();
		IndicatorDTO indicator = iIndicatorService.getById(id);

		return R.ok().data(indicator);
	}

	/**
	 * 添加
	 *
	 * @param indicator
	 * @return
	 */

	//@ImsPreAuth("eval:dataDictionary:edit")
	@PostMapping(value = "/save")
	@ApiOperation(value = "新增(修改)", notes = "新增（修改）")
	public R addAll(@RequestBody Indicator indicator) {

		try {
			boolean b = iIndicatorService.saveOrUpdate(indicator);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error().data("保存失败！");
			}
		} catch (CustomException e){
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 添加
	 *
	 * @param indicator
	 * @return
	 */

	//@ImsPreAuth("eval:dataDictionary:edit")
	@PostMapping(value = "/saveGroup")
	@ApiOperation(value = "新增(修改)", notes = "新增（修改）")
	public R saveGroup(@RequestBody IndicatorDTO indicator) {

		try {
			boolean b = iIndicatorService.saveGroup(indicator);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error().data("保存失败！");
			}
		} catch (CustomException e){
			log.error("错误",e);
			return R.customError(e.getMessage()).data("失败！");
		}
	}




	/**
	 * 批量删除
	 *
	 * @param ids
	 * @return
	 */
	//@ImsPreAuth("eval:dataDictionary:remove")
	@PostMapping(value = "/remove/{ids}")
	@ApiOperation(value = "删除", notes = "删除")
	public R deleteAll(@PathVariable("ids") String ids) {
		String[] strings = ids.split(",");
		boolean b = iIndicatorService.removeByIds(Arrays.asList(strings));
		if (b) {
			return R.ok().data(b);
		} else {
			return R.error().data("删除失败！");
		}
	}




}
