package com.ims.eval.controller;


import com.ims.eval.config.CustomException;
import com.ims.eval.entity.ResponsibilityIndicatorInfo;
import com.ims.eval.entity.dto.request.ResponsibilityIndicatorInfoUpdateDTO;
import com.ims.eval.entity.dto.result.R;
import com.ims.eval.service.IResponsibilityIndicatorInfoService;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 责任书指标明细

 前端控制器
 * </p>
 *
 * @author wang
 * @since 2023-03-08
 */
@Slf4j
@RestController
@RequestMapping("//responsibility-indicator-info")
public class ResponsibilityIndicatorInfoController {



	@Autowired
	private IResponsibilityIndicatorInfoService responsibilityIndicatorInfoService;


	@Autowired
	private HttpServletRequest request;

	/**
	 *
	 * @param deptResponsibilityIds 目标责任书id
	 * @return
	 */
	//@ImsPreAuth("eval:responsibilityIndicatorInfo:view")
	@GetMapping(value = "planValueList")
	public R planValueList(
				  @RequestParam(value = "deptResponsibilityIds", required = false) List<String> deptResponsibilityIds,
				  @RequestParam(value = "dept", required = false) String dept) {
		try {
			Map list = responsibilityIndicatorInfoService.planValueList(deptResponsibilityIds, dept,request);
			return R.ok().data(list);
		} catch (CustomException e) {
			e.printStackTrace();
			return R.customError(e.getMessage());
		}
	}


	/**
	 *
	 * @param deptResponsibilityIds 目标责任书id
	 * @return
	 */
	//@ImsPreAuth("eval:responsibilityIndicatorInfo:view")
	@GetMapping(value = "finishValueList")
	public R finishValueList(
		@RequestParam(value = "deptResponsibilityIds", required = false) List<String> deptResponsibilityIds,
		@RequestParam(value = "dept", required = false) String dept) {
		Map list = responsibilityIndicatorInfoService.finishValueList(deptResponsibilityIds, dept);
		return R.ok().data(list);
	}


	/**
	 * 添加
	 *
	 * @param indicatorInfo
	 * @return
	 */

	//@ImsPreAuth("eval:responsibilityIndicatorInfo:edit")
	@PostMapping(value = "/save")
	@ApiOperation(value = "新增(修改)", notes = "新增（修改）")
	public R add(@RequestBody ResponsibilityIndicatorInfo indicatorInfo) {

		try {
			boolean b = responsibilityIndicatorInfoService.saveOrUpdate(indicatorInfo);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error().data("保存失败！");
			}
		} catch (CustomException e){
			return R.customError(e.getMessage()).data("失败！");
		}
	}


	/**
	 * 修改
	 *
	 * @param indicatorInfos
	 * @return
	 */

	//@ImsPreAuth("eval:responsibilityIndicatorInfo:edit")
	@PostMapping(value = "/saveDto")
	@ApiOperation(value = "新增(修改)", notes = "新增（修改）")
	public R saveDto(@RequestBody ResponsibilityIndicatorInfoUpdateDTO indicatorInfos) {

		try {
			boolean b = responsibilityIndicatorInfoService.saveOrUpdateDto(indicatorInfos);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error().data("保存失败！");
			}
		} catch (CustomException e){
			return R.customError(e.getMessage()).data("失败！");
		}
	}


	/**
	 * 批量修改
	 *
	 * @param indicatorInfos
	 * @return
	 */

	//@ImsPreAuth("eval:responsibilityIndicatorInfo:edit")
	@PostMapping(value = "/saveBatchDto")
	@ApiOperation(value = "批量新增(修改)", notes = "批量新增（修改）")
	public R saveBatchDto(@RequestBody List<ResponsibilityIndicatorInfoUpdateDTO> indicatorInfos) {

		try {
			boolean b = responsibilityIndicatorInfoService.saveBatchDto(indicatorInfos);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error().data("保存失败！");
			}
		} catch (CustomException e){
			return R.customError(e.getMessage()).data("失败！");
		}
	}

}
