package com.ims.eval.schedule;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ims.common.utils.StringUtils;
import com.ims.eval.dao.*;
import com.ims.eval.entity.*;
import com.ims.eval.entity.dto.response.NeatDTo;
import com.ims.eval.expression.*;
import com.ims.eval.service.IEvaluationScoreCountService;
import com.ims.eval.util.MathCalculatorUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author ：xugp
 * @date ：Created in 2023/3/15 9:49
 * @description：得分规则计算
 * @modified By：
 * @version: $
 */

@Slf4j
@Component
@EnableScheduling
public class ScoreCalculationSchedule {

	@Autowired
	private OrganizationEvaluationInfoMapper organizationEvaluationInfoMapper;

	@Autowired
	private EvaluationScoringRuleMapper evaluationScoringRuleMapper;

	@Autowired
	private IntervalScoringTableMapper intervalScoringTableMapper;

	@Autowired
	private IndicatorMapper indicatorMapper;

	@Autowired
	private OrganizationEvaluationMapper organizationEvaluationMapper;

	@Autowired
	private IEvaluationScoreCountService iEvaluatioinScoreCountService;

	@Autowired
	private CalculateIndicatorItemInfoMapper calculateIndicatorItemInfoMapper;

	/*@Scheduled(cron = "* 10 11 ? * 3")*/
	public boolean doTask(String id){

		boolean save = true;
		try {

			//判断本次考评的计算记录是否存在  如果存在，删除计算记录
			QueryWrapper<EvaluationScoreCount> qw1 = new QueryWrapper<>();
			if (StringUtils.isNotEmpty(id)) {
				qw1.lambda().eq(EvaluationScoreCount::getOrganizationEvaluationId, id);
			}
			qw1.lambda().eq(EvaluationScoreCount::getIsQuantified, "1");
			List<EvaluationScoreCount> evaluationScoreCountList = iEvaluatioinScoreCountService.list(qw1);
			if (evaluationScoreCountList.size() >0){
				List<String> idList = evaluationScoreCountList.stream().map(EvaluationScoreCount::getId).collect(Collectors.toList());
				iEvaluatioinScoreCountService.removeByIds(idList);
			}

			//加载单位/部门考评记录
			//todo 后续添加查询条件
			OrganizationEvaluation organizationEvaluation = organizationEvaluationMapper.selectById(id);
			if (organizationEvaluation == null){
				throw new Exception("考评记录为空！！！");
			}
			//加载该考评记录下的考评指标明细数据
			List<String> list = Arrays.asList(organizationEvaluation.getOrganizationEvaluationRuleId().split(","));

			QueryWrapper<OrganizationEvaluationInfo> qw = new QueryWrapper<>();
			if (StringUtils.isNotEmpty(organizationEvaluation.getOrganizationEvaluationRuleId())) {
				qw.lambda().in(OrganizationEvaluationInfo::getOrganizationEvaluationRuleId,list);
			}
			List<OrganizationEvaluationInfo> organizationEvaluationInfos= organizationEvaluationInfoMapper.selectList(qw);
			if (organizationEvaluationInfos.size() > 0){
				//对考评规则id进行分组
				Map<String, List<OrganizationEvaluationInfo>> collect = organizationEvaluationInfos.stream().collect(Collectors.groupingBy(OrganizationEvaluationInfo::getOrganizationEvaluationRuleId));
				for (Map.Entry<String, List<OrganizationEvaluationInfo>> entry : collect.entrySet()){
					//对指标id进行分组
					if (entry.getValue().size() > 0){
						Map<String, List<OrganizationEvaluationInfo>> collectIn = entry.getValue().stream().collect(Collectors.groupingBy(OrganizationEvaluationInfo::getIndicatorId));
						for (Map.Entry<String, List<OrganizationEvaluationInfo>> listEntry : collectIn.entrySet()){
							Indicator indicator = indicatorMapper.selectById(listEntry.getKey());
							QueryWrapper<EvaluationScoringRule> e = new QueryWrapper<>();
							if (StringUtils.isNotEmpty(indicator.getIndicatorCode())) {
								e.lambda().eq(EvaluationScoringRule::getIndicatorId,indicator.getIndicatorCode());
							}
							EvaluationScoringRule evaluationScoringRule = evaluationScoringRuleMapper.selectOne(e);
							if (evaluationScoringRule != null){
								String calculation = replace(listEntry.getValue(), evaluationScoringRule);
								log.info(indicator.getIndicatorCode()+"得分为" + calculation);
								//将得分存入考评得分统计表
								EvaluationScoreCount evaluationScoreCount = EvaluationScoreCount.builder()
									.indicatorId(listEntry.getKey())
									.organizationEvaluationRuleId(entry.getKey())
									.organizationEvaluationId(organizationEvaluation.getId())
									.score(Double.parseDouble(calculation))
									.isQuantified("1")
									.obversionScore(Double.parseDouble(calculation))
									.build();
								save = iEvaluatioinScoreCountService.save(evaluationScoreCount);
							}
						}
					}
				}

			}else {
				throw new Exception("考评指标明细数据为空！！");
			}

		}catch (Exception e){
			log.info(e.getMessage());
		}
		return save;
	}

	//中间变量替换为数据
	public String replace(List<OrganizationEvaluationInfo> list,EvaluationScoringRule evaluationScoringRule) throws Exception {

		String score = null;
		//判断可用量化的进行变量替换
		if ("是".equals(list.get(0).getIsQuantified())){

			//删除考评指标明细中的包含max或者min的指标项
			Iterator<OrganizationEvaluationInfo> iterator = list.iterator();
			while (iterator.hasNext()){
				OrganizationEvaluationInfo next = iterator.next();
				if (next.getOptionCode().contains("MAX")) {
					iterator.remove();
				}else if (next.getOptionCode().contains("MIN")){
					iterator.remove();
				}
			}

			List<NeatDTo> dToList = new ArrayList<>();
			list.stream().forEach(r ->{
				NeatDTo neatDTo = new NeatDTo();
				neatDTo.setCode(r.getOptionCode());
				neatDTo.setValue(r.getQuantifiedValue());
				neatDTo.setLength(r.getOptionCode().length());
				dToList.add(neatDTo);
			});
			//添加计算指标项的code和值  例如：差值最大值和最小值
			OrganizationEvaluation organizationEvaluation = organizationEvaluationMapper.selectById(list.get(0).getOrganizationEvaluationId());
			List<CalculateIndicatorItemInfo> calculateIndicatorItemInfos = calculateIndicatorItemInfoMapper.selectList(organizationEvaluation.getOrganizationType(),organizationEvaluation.getCheckCycle(),organizationEvaluation.getYear(),organizationEvaluation.getMonth());
			calculateIndicatorItemInfos.stream().forEach(r ->{
				NeatDTo neatDTo = new NeatDTo();
				neatDTo.setCode(r.getOptionCode());
				neatDTo.setValue(r.getQuantifiedValue());
				neatDTo.setLength(r.getOptionCode().length());
				dToList.add(neatDTo);
			});
			//处理code长度从大到小排序
			List<NeatDTo> collect = dToList.stream().sorted(Comparator.comparing(NeatDTo::getLength).reversed()).collect(Collectors.toList());

			//查询得分规则公式
			QueryWrapper<IntervalScoringTable> qw = new QueryWrapper<>();
			if (StringUtils.isNotEmpty(evaluationScoringRule.getId())) {
				qw.lambda().eq(IntervalScoringTable::getRuleId,evaluationScoringRule.getId());
			}
			List<IntervalScoringTable> intervalScoringTables = intervalScoringTableMapper.selectList(qw);
			for (NeatDTo neatDTo : collect){
				for (IntervalScoringTable r : intervalScoringTables){
					r.setRange(r.getRange().replaceAll(neatDTo.getCode(), neatDTo.getValue() + ""));
					r.setRegularExpression(r.getRegularExpression().replaceAll(neatDTo.getCode(),neatDTo.getValue() + ""));
				}
			}
			//针对只有一种区间评分
			if (intervalScoringTables.size() == 1){
				String regularExpression = intervalScoringTables.get(0).getRegularExpression();
				log.info(evaluationScoringRule.getScoreRuleName()+ "公式为" + regularExpression);
				//公式计算得分
				score = MathCalculatorUtil.handle(regularExpression.trim());
				if (!StringUtils.isNotEmpty(score)){
					score = "0";
				}
			} else if(intervalScoringTables.size() > 1){
				//针对多区间评分
				for (IntervalScoringTable intervalScoringTable : intervalScoringTables){
					String range = intervalScoringTable.getRange();
					if (rangeHandle(range)){
						score =  MathCalculatorUtil.handle(intervalScoringTable.getRegularExpression().trim());
						if (!StringUtils.isNotEmpty(score)){
							score = "0";
						}
						break;
					}
				}
			}
		}else {
			throw new Exception("该指标是非量化指标！");
		}
		return score;
	}


	/*
	 *  判断是否在该区间
	 * */
	public Boolean rangeHandle(String range) {

		Boolean s = true;
		try {
			//将区间表达式拆分
			ArrayList<Token> tokens = Analyzer.getTokens(range);
			//使用不同编码代替函数  支持三级函数嵌套
			AlarmExpression alarmExpression = Analyzer.createAlarmExpression(tokens);
			if (alarmExpression.getVarList() != null && alarmExpression.getVarList().size() > 0) {
				List<AlarmExpression> alarmExpressions = alarmExpression.getChildren();
				for (AlarmExpression alarmExpression1 : alarmExpressions){
					if (alarmExpression1.getChildren().size() > 0 && alarmExpression1.getChildren() != null){
						for (AlarmExpression alarmExpression2 : alarmExpression1.getChildren()){
							if (alarmExpression2.getChildren().size() > 0 && alarmExpression2.getChildren() != null){
								List<AlarmExpression> al = alarmExpression2.getChildren();
								for (AlarmExpression alarmExpression3 : al){
									String calculator = MathCalculatorUtil.calculator(alarmExpression3.getExpression());
									alarmExpression2.setExpression(alarmExpression2.getExpression().replaceAll(alarmExpression3.getFunCode(), calculator));
								}
							}
							String calculator = MathCalculatorUtil.calculator(alarmExpression2.getExpression());
							alarmExpression1.setExpression(alarmExpression1.getExpression().replaceAll(alarmExpression2.getFunCode(), calculator));
						}
						String calculator = MathCalculatorUtil.calculator(alarmExpression1.getExpression());
						alarmExpression.setExpression(alarmExpression.getExpression().replaceAll(alarmExpression1.getFunCode(), calculator));
					}else {
						String calculator = MathCalculatorUtil.calculator(alarmExpression1.getExpression());
						alarmExpression.setExpression(alarmExpression.getExpression().replaceAll(alarmExpression1.getFunCode(), calculator));
					}
				}
				Map<String,Object> map = new HashMap<>();
				s = (Boolean) ScriptShell.parseExpr(alarmExpression.getExpression(), map);
			}
		}catch (Exception e){
			log.info(e.getMessage());
		}
		return s;
	}



//	public static void main(String[] args) {
//
//
//		String sc = "if(10+1*2/maxnum(1)>12,12,if(1<0,10-1*2/minnum(1),10+1*2/maxnum(1)))";
//
//
//		String s2 = "if(1>0,-1*2/maxnum(2)+10,1*2/minnum(1)+10)";
//
//		String s4 = "if(0.01<0,-0.01*2/min(1,0.07)+10,1/1*2/max(4,0.07)+10)";
//		String s5 = "1+if(if(1<0,-1*2/min(2,1)+10,1*2/max(2,1)+10)>12,12,if(1<0,-1*2/min(2,1)+10,1*2/max(2,1)+10))";
//		String s3 = "if(abs(99)<=5%,if(abs(99)>=1%,(5%-abs(99))/1%*0.25+5,6),if(abs(99)<=10%,(5%-abs(99))/1%*0.2+5,4))+if(99==0,5,if(99>0,6,4))";
//		/*String sd = "max(1,min(1,2))";
//		String sc = MathCalculatorUtil.calculator(sd);*/
//		String ss = "if(max(1,3)<1,1,2)";
//		String s9 = "maxmin()";
//		ArrayList<Token> tokens = Analyzer.getTokens(sc);
//
//		//使用不同编码代替函数
//		AlarmExpression alarmExpression = Analyzer.createAlarmExpression(tokens);
//		if (alarmExpression.getVarList() != null && alarmExpression.getVarList().size() > 0) {
//			List<AlarmExpression> alarmExpressions = alarmExpression.getChildren();
//			for (AlarmExpression alarmExpression1 : alarmExpressions){
//				if (alarmExpression1.getChildren().size() > 0 && alarmExpression1.getChildren() != null){
//					for (AlarmExpression alarmExpression2 : alarmExpression1.getChildren()){
//						if (alarmExpression2.getChildren().size() > 0 && alarmExpression2.getChildren() != null){
//							List<AlarmExpression> al = alarmExpression2.getChildren();
//							for (AlarmExpression alarmExpression3 : al){
//								String calculator = MathCalculatorUtil.calculator(alarmExpression3.getExpression());
//								alarmExpression2.setExpression(alarmExpression2.getExpression().replaceAll(alarmExpression3.getFunCode(), calculator));
//							}
//						}
//						String calculator = MathCalculatorUtil.calculator(alarmExpression2.getExpression());
//						alarmExpression1.setExpression(alarmExpression1.getExpression().replaceAll(alarmExpression2.getFunCode(), calculator));
//					}
//					String calculator = MathCalculatorUtil.calculator(alarmExpression1.getExpression());
//					alarmExpression.setExpression(alarmExpression.getExpression().replaceAll(alarmExpression1.getFunCode(), calculator));
//				}else {
//					String calculator = MathCalculatorUtil.calculator(alarmExpression1.getExpression());
//					alarmExpression.setExpression(alarmExpression.getExpression().replaceAll(alarmExpression1.getFunCode(), calculator));
//				}
//			}
//		}
////		Map<String,Object> map = new HashMap<>();
////		Object o = ScriptShell.parseExpr(alarmExpression.getExpression(), map);
////		System.out.println(o);
//		//判断表达式返回结果
//		String calculator = MathCalculatorUtil.calculator(alarmExpression.getExpression());
//		System.out.println(calculator);
//
//	}

}
