package com.ims.eval.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.BinSection;
import com.ims.eval.dao.BinSectionMapper;
import com.ims.eval.service.IBinSectionService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * <p>
 * 生产经营业务分类 服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-03-12
 */
@Service
public class BinSectionServiceImpl extends ServiceImpl<BinSectionMapper, BinSection> implements IBinSectionService {

	@Override
	public List<BinSection> list(String id, String sectionName, String sectionCode) {

		QueryWrapper<BinSection> qw = new QueryWrapper();
		qw.lambda().eq(BinSection::getId, id);
		qw.lambda().like(BinSection::getSectionName, sectionName);
		qw.lambda().eq(BinSection::getSectionCode, sectionCode);
		qw.lambda().orderByAsc(BinSection::getOrderNum);
		List<BinSection> list = baseMapper.selectList(qw);

		return list;
	}

	@Transactional
	@Override
	public boolean saveOrUpdate(BinSection entity) {

		if (null != entity && (null != entity.getId() && !"".equals(entity.getId().trim()))) {
			BinSection type = baseMapper.selectById(entity.getId());
			type.setDelFlag(true);
			baseMapper.updateById(type);
		}

		QueryWrapper<BinSection> qw = new QueryWrapper();
		qw.lambda().eq(BinSection::getSectionCode, entity.getSectionCode());
		qw.lambda().eq(BinSection::getDelFlag, false);
		List<BinSection> list = baseMapper.selectList(qw);
		if (null != list && list.size() > 0) {
			throw new CustomException("当前分类已存在");
		}

		int count = baseMapper.selectCount(null);
		entity.setId(null);
		entity.setOrderNum(count);
		entity.setDelFlag(false);
		return super.saveOrUpdate(entity);
	}

}
