package com.ims.eval.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ims.eval.dao.*;
import com.ims.eval.entity.CalculateIndicatorItemInfo;
import com.ims.eval.entity.OrganizationEvaluationInfo;
import com.ims.eval.entity.dto.response.OrganizationEvaluationResDTO;
import com.ims.eval.service.ICalculateIndicatorItemInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

import static org.codehaus.groovy.runtime.DefaultGroovyMethods.collect;

/**
 * <p>
 * 计算指标项明细 服务实现类
 * </p>
 *
 * @author xugp
 * @since 2023-03-26
 */
@Service
public class CalculateIndicatorItemInfoServiceImpl extends ServiceImpl<CalculateIndicatorItemInfoMapper, CalculateIndicatorItemInfo> implements ICalculateIndicatorItemInfoService {



	@Autowired
	private OrganizationEvaluationMapper organizationEvaluationMapper;

	@Autowired
	private CalculateIndicatorItemInfoMapper calculateIndicatorItemInfoMapper;

	@Autowired
	private IndicatorMapper indicatorMapper;


	@Override
	public int saveIndicatorInfo(List<OrganizationEvaluationInfo> organizationEvaluationInfos) throws Exception {

		List<CalculateIndicatorItemInfo> list = new ArrayList<>();

		OrganizationEvaluationResDTO organizationEvaluationResDTO = organizationEvaluationMapper.selectById(organizationEvaluationInfos.get(0).getOrganizationEvaluationId());
		//对考评规则id进行分组
		Map<String, List<OrganizationEvaluationInfo>> collect = organizationEvaluationInfos.stream().collect(Collectors.groupingBy(OrganizationEvaluationInfo::getOrganizationEvaluationRuleId));
		for (Map.Entry<String, List<OrganizationEvaluationInfo>> entry : collect.entrySet()) {
			if (entry.getValue().size() > 0) {
				//对指标id进行分组
				Map<String, List<OrganizationEvaluationInfo>> collectIn = entry.getValue().stream().collect(Collectors.groupingBy(OrganizationEvaluationInfo::getIndicatorId));
				for (Map.Entry<String, List<OrganizationEvaluationInfo>> listEntry : collectIn.entrySet()) {
					List<OrganizationEvaluationInfo> listEntryValue = listEntry.getValue();
					for (OrganizationEvaluationInfo r : listEntryValue){
						if (r.getOptionCode().equals("CZ") || r.getOptionCode().equals("LRGXL") || r.getOptionCode().equals("DWQWLR")) {
							CalculateIndicatorItemInfo calculateIndicatorItemInfo = new CalculateIndicatorItemInfo();
							calculateIndicatorItemInfo.setIndicatorId(listEntry.getKey());
							calculateIndicatorItemInfo.setOptionCode(r.getOptionCode());
							calculateIndicatorItemInfo.setQuantifiedValue(r.getQuantifiedValue());
							calculateIndicatorItemInfo.setSectionId(indicatorMapper.selectById(listEntry.getKey()).getBinSection());
							calculateIndicatorItemInfo.setOrganizationType(organizationEvaluationResDTO.getOrganizationType());
							calculateIndicatorItemInfo.setCheckCycle(organizationEvaluationResDTO.getCheckCycle());
							calculateIndicatorItemInfo.setYear(organizationEvaluationResDTO.getYear());
							calculateIndicatorItemInfo.setMonth(organizationEvaluationResDTO.getMonth());
							list.add(calculateIndicatorItemInfo);
						}
					}
				}
			}
		}

		return saveCalculate(list);
	}


	@Transactional
	public int saveCalculate(List<CalculateIndicatorItemInfo> list) throws Exception {

		if (list.size() > 0){

			//判断计算指标明细中之前是否存在记录 如果存在删除之前记录,重新新增
			List<CalculateIndicatorItemInfo> calculateIndicatorItemInfo = calculateIndicatorItemInfoMapper.selectList(list.get(0).getOrganizationType(),list.get(0).getCheckCycle(),list.get(0).getYear(),list.get(0).getMonth());

			if (calculateIndicatorItemInfo.size() > 0){
				List<String> idList = calculateIndicatorItemInfo.stream().map(CalculateIndicatorItemInfo::getId).collect(Collectors.toList());
				calculateIndicatorItemInfoMapper.deleteBatchIds(idList);
			}

			//遍历list解析最大值和最小值
			Map<String, List<CalculateIndicatorItemInfo>> collect = list.stream().collect(Collectors.groupingBy(CalculateIndicatorItemInfo::getOptionCode));
			for (Map.Entry<String, List<CalculateIndicatorItemInfo>> listEntry : collect.entrySet()){

				//从小到大的顺序排序
				Collections.sort(listEntry.getValue(), new Comparator<CalculateIndicatorItemInfo>() {
					@Override
					public int compare(CalculateIndicatorItemInfo u1, CalculateIndicatorItemInfo u2) {
						double diff = u1.getQuantifiedValue() - u2.getQuantifiedValue();
						BigDecimal two = new BigDecimal(diff);
						double three = two.setScale(2,BigDecimal.ROUND_HALF_UP).doubleValue();
						if (three > 0) {
							return 1;
						} else if (three < 0) {
							return -1;
						}
						return 0; //相等为0
					}
				});

				CalculateIndicatorItemInfo maxNum = listEntry.getValue().get(listEntry.getValue().size()-1);
				maxNum.setMark("1");

				CalculateIndicatorItemInfo minNum = listEntry.getValue().get(0);
				minNum.setMark("0");


				if (listEntry.getKey().equals("CZ")){
					//存储最大值
					maxNum.setOptionCode("CZMAX");
					minNum.setOptionCode("CZMIN");
				}else if (listEntry.getKey().equals("LRGXL")){
					maxNum.setOptionCode("LRGXKLMAX");
					minNum.setOptionCode("LRGXKLMIN");
				}else if (listEntry.getKey().equals("DWQWLR")){
					maxNum.setOptionCode("DWQWLRMAX");
					minNum.setOptionCode("DWQWLRMIN");
				}
				int insert = calculateIndicatorItemInfoMapper.insert(maxNum);

				int insert1 = calculateIndicatorItemInfoMapper.insert(minNum);
				System.out.println(insert1);
			}
		}else {
			throw new Exception("计算指标项明细为空");
		}
		return 1;
	}
}
