package com.ims.eval.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ims.common.utils.DateUtils;
import com.ims.common.utils.StringUtils;
import com.ims.eval.config.CustomException;
import com.ims.eval.dao.ResponsibilityIndicatorInfoMapper;
import com.ims.eval.entity.*;
import com.ims.eval.dao.DeptResponsibilityMapper;
import com.ims.eval.entity.dto.response.DeptResponsibilityResDTO;
import com.ims.eval.entity.dto.response.ResponsibilityIndicatorInfoResDTO;
import com.ims.eval.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.Serializable;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 部门目标责任表 服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-02-27
 */
@Service
public class DeptResponsibilityServiceImpl extends ServiceImpl<DeptResponsibilityMapper, DeptResponsibility> implements IDeptResponsibilityService {


	/**
	 * 指标主业务类
	 */
	@Autowired
	private IIndicatorService iIndicatorService;

	/**
	 * 指标子业务类
	 */
	@Autowired
	private IIndicatorDictionaryService indicatorDictionaryService;


	/**
	 * 单位/部门考评配置业务类
	 */
	@Autowired
	private IOrganizationEvaluationRuleService organizationEvaluationRuleService;

	/**
	 * 考评规则业务类
	 */
	@Autowired
	private IEvaluateRuleService evaluateRuleService;

	/**
	 * 考评规则明细业务类
	 */
	@Autowired
	private IEvaluateRuleInfoService evaluateRuleInfoService;


	@Resource
	private ResponsibilityIndicatorInfoMapper responsibilityIndicatorInfoMapper;

	@Resource
	private IResponsibilityIndicatorInfoService responsibilityIndicatorInfoService;


	@Autowired
	private IUserService userService;

	@Override
	public IPage<DeptResponsibility> list(Integer pageNum, Integer pageSize, String id, String responsibilityCode,String cycleUnit,  List<String> checkCycle, String beginDate, String endDate, String stage, String createBy, String year, String month,String des) {

		QueryWrapper<DeptResponsibility> qw = new QueryWrapper<>();
		if (null == pageNum || null == pageSize) {
			throw new CustomException("分页参数为空");
		}
		//构造分页构造器
		Page<DeptResponsibility> page = new Page<>(pageNum, pageSize);

		if (StringUtils.isNotEmpty(id)) {
			qw.lambda().eq(DeptResponsibility::getId, id);
		}

		if (StringUtils.isNotEmpty(responsibilityCode)) {
			qw.lambda().like(DeptResponsibility::getResponsibilityCode, responsibilityCode);
		}

		if (StringUtils.isNotEmpty(cycleUnit)) {
			qw.lambda().eq(DeptResponsibility::getCycleUnit, cycleUnit);
		}

		if (StringUtils.isNotEmpty(checkCycle)) {
			qw.lambda().in(DeptResponsibility::getCheckCycle, checkCycle);
		}

		if (StringUtils.isNotEmpty(beginDate) && StringUtils.isNotEmpty(endDate)) {
			qw.lambda().ge(DeptResponsibility::getBeginDate, beginDate);
			qw.lambda().le(DeptResponsibility::getEndDate, endDate);
		}

		if (StringUtils.isNotEmpty(stage)) {
			qw.lambda().eq(DeptResponsibility::getStage, stage);
		}

		if (StringUtils.isNotEmpty(createBy)) {
			qw.lambda().eq(DeptResponsibility::getCreateBy, createBy);
		}

		if (StringUtils.isNotEmpty(createBy)) {
			qw.lambda().eq(DeptResponsibility::getCreateBy, createBy);
		}
		if (StringUtils.isNotEmpty(year)) {
			qw.lambda().eq(DeptResponsibility::getYear, year);
		}
		if (StringUtils.isNotEmpty(month)) {
			qw.lambda().eq(DeptResponsibility::getMonth, month);
		}

		if (StringUtils.isNotEmpty(des)) {
			qw.lambda().like(DeptResponsibility::getDes, des);
		}

		qw.lambda().orderByDesc(DeptResponsibility::getCreateTime);

		IPage<DeptResponsibility> list = baseMapper.selectPage(page, qw);
		return list;
	}


	@Transactional
	@Override
	public boolean generateResponsibility(String id) {


		DeptResponsibility oriResponsibility = baseMapper.selectById(id);
		if (null == oriResponsibility) {
			throw new CustomException("此目标责任书不存在");
		}

		QueryWrapper<ResponsibilityIndicatorInfo> infoqw = new QueryWrapper<>();
		infoqw.lambda().eq(ResponsibilityIndicatorInfo::getDeptResponsibilityId, id);
		int infcount = responsibilityIndicatorInfoService.count(infoqw);
		if (infcount > 0) {
			throw new CustomException("编号" + oriResponsibility.getResponsibilityCode() + "的责任书已启动;不允许重复操作");
		}

		oriResponsibility.setStage("流程启动");
		baseMapper.updateById(oriResponsibility);

		List<String> evaluationRuleIds = Arrays.asList(oriResponsibility.getOrganizationEvaluationRuleId().split(","));

		for (String ruleId : evaluationRuleIds) {
			//获取指定部门的规则
			OrganizationEvaluationRule evaluationRule = organizationEvaluationRuleService.getById(ruleId);
			if (null == evaluationRule) {
				throw new CustomException("不存在此单位权重配置");
			}

			//获取对应的考评规（至少存在一条）
			List<String> evaluateRuleIds = Arrays.asList(evaluationRule.getEvaluateRuleId().split(","));
			List<EvaluateRule> evaluateRuleList = evaluateRuleService.listAll(evaluateRuleIds);
			if (null == evaluateRuleList || evaluateRuleList.size() <= 0) {
				throw new CustomException("未获取到配置的规则");
			}
			//获取考评规则的id
			List<String> ruleIds = evaluateRuleList.stream().map(EvaluateRule::getId).collect(Collectors.toList());
			//通过规则id获取对应的规则明细
			List<EvaluateRuleInfo> ruleInfos = evaluateRuleInfoService.list(ruleIds);
			if (null == ruleInfos || ruleInfos.size() <= 0) {
				throw new CustomException("规则明细配置为空");
			}


			//获取考评规则中的指标id
			List<String> indicatorIds = ruleInfos.stream().map(EvaluateRuleInfo::getIndicatorId).collect(Collectors.toList());

			//获取指标对象
			List<Indicator>  indicators = iIndicatorService.selectByIds(indicatorIds);

			/***   目标目标责任书默认生成目标值 ZRMB     ***/
			indicators.stream().forEach(i -> {
				ResponsibilityIndicatorInfo info = new ResponsibilityIndicatorInfo();
				info.setDeptResponsibilityId(oriResponsibility.getId());
				info.setOrganizationEvaluationRuleId(ruleId);
				info.setIndicatorId(i.getId());
				info.setIndicatorDictionaryId(null);
				info.setOptionCode("ZRMB");
				info.setIsQuantified(i.getIsQuantified());
				info.setCreateTime(new Date());
				boolean b2 = responsibilityIndicatorInfoService.saveOrUpdate(info);
				if (!b2) {
					throw new CustomException("初始化目标责任书明细失败");
				}
			});


			//通过指标id获取指标明细项
/*			List<IndicatorResDTO> resDTOS = iIndicatorService.listByIds(indicatorIds);
			if (null == resDTOS || resDTOS.size() <= 0) {
				throw new CustomException("指标规则明细项为空");
			}


			resDTOS.stream().forEach(r -> {
				ResponsibilityIndicatorInfo info = new ResponsibilityIndicatorInfo();
				info.setDeptResponsibilityId(oriResponsibility.getId());
				info.setIndicatorId(r.getId());
				info.setIndicatorDictionaryId(r.getIndicatorDictionaryID());
				info.setOptionCode(r.getOptionCode());
				info.setIsQuantified(r.getIsQuantified());
				info.setCreateTime(new Date());
				boolean b2 = responsibilityIndicatorInfoService.saveOrUpdate(info);
				if (!b2) {
					throw new CustomException("初始化目标责任书明细失败");
				}
			});*/
		}

		return true;
	}

	@Override
	public DeptResponsibilityResDTO getByidAndInfo(String id, String dept,HttpServletRequest request) {
		DeptResponsibilityResDTO resDTO = baseMapper.selectById(id);

		Map<String, List<ResponsibilityIndicatorInfoResDTO>> map = responsibilityIndicatorInfoService.planValueList(Arrays.asList(id), dept,request);
		if(null != resDTO){
			resDTO.setMap(map);
		}

		return resDTO;
	}

	@Override
	public List<Map> targetValueeport(String id, String dept, HttpServletRequest request) {
		DeptResponsibilityResDTO resDTO = baseMapper.selectById(id);
		Myuser myuser = userService.getSysUser(request.getHeader("code"));
//		responsibilityIndicatorInfoMapper.listByresponsibilityId()

		return null;
	}

	@Override
	public boolean editState(String id, String state,String instId) {
		DeptResponsibility responsibility = baseMapper.selectById(id);
		if (StringUtils.isNotEmpty(state)) {
			responsibility.setStage(state);
		}
		if (StringUtils.isNotEmpty(instId)) {
			responsibility.setInstId(instId);
		}
		boolean b = super.saveOrUpdate(responsibility);
		return b;
	}

	@Transactional
	@Override
	public boolean saveOrUpdate(DeptResponsibility entity) {
		//获取指定部门的规则
		List<OrganizationEvaluationRule> list = organizationEvaluationRuleService.listByIsCheck(entity.getOrganizationType(), entity.getCheckCycle(), true, false);
		if (null == list || list.size() <= 0) {
			throw new CustomException("不存在单位权重配置");
		}
		//获取组织id;
		List<String> organizationIds = list.stream().map(OrganizationEvaluationRule::getOrganizationId).collect(Collectors.toList());
		List<String> organizationNames = list.stream().map(OrganizationEvaluationRule::getOrganizationName).collect(Collectors.toList());
		List<String> evaluationRuleIds = list.stream().map(OrganizationEvaluationRule::getId).collect(Collectors.toList());

		StringBuilder responsibilityCode = new StringBuilder();
		responsibilityCode.append("mbzr_");
		responsibilityCode.append(entity.getCheckCycle()).append("_");

		//判断该是否存在目标责任书
		List<DeptResponsibility> oriList = new ArrayList<>();
		if ("NDKP".equals(entity.getCheckCycle())) {
			oriList = list("", entity.getCheckCycle(), entity.getYear(), "");
			responsibilityCode.append(entity.getYear()).append("_");
			responsibilityCode.append(StringUtils.addZeroForStr(String.valueOf(oriList.size()+1), 4, "l"));
		}
		if ("JDKP".equals(entity.getCheckCycle())) {
			Date newDate = DateUtils.toDate(entity.getYear() + "-" + entity.getMonth() + "-01");
			String season = String.valueOf(DateUtils.getSeason(newDate));//获取季度
			String month = String.valueOf(Integer.valueOf(entity.getMonth()) * Integer.valueOf(season));//季度考评将
			oriList = list("", entity.getCheckCycle(), entity.getYear(), month);
			List<DeptResponsibility> count = list("", entity.getCheckCycle(), entity.getYear(), "");
			responsibilityCode.append(entity.getYear()).append("_").append(month).append("_");
			responsibilityCode.append(StringUtils.addZeroForStr(String.valueOf(count.size()+1), 4, "l"));
		}
		if ("YDKP".equals(entity.getCheckCycle())) {
			oriList = list("", entity.getCheckCycle(), entity.getYear(), entity.getMonth());
			responsibilityCode.append(entity.getYear()).append("_").append(entity.getMonth()).append("_");
			List<DeptResponsibility> count = list("", entity.getCheckCycle(), entity.getYear(), "");
			responsibilityCode.append(StringUtils.addZeroForStr(String.valueOf(count.size()+1), 4, "l"));
		}


		if (null != entity && (null == entity.getId() || "".equals(entity.getId().trim()))) {
			if (null != oriList && oriList.size() > 0) {
				throw new CustomException("此单位在该阶段已存在目标责任书");
			}
			//保存目标责任书
			entity.setResponsibilityCode(responsibilityCode.toString().toLowerCase());

			entity.setOrganizationId(String.join(",", organizationIds));//考评组织ID
			entity.setOrganizationName(String.join(",", organizationNames));
			entity.setOrganizationEvaluationRuleId(String.join(",", evaluationRuleIds));
			entity.setStage("流程未启动");
			entity.setCreateTime(new Date());
		} else {
			DeptResponsibility update = baseMapper.selectById(entity.getId());
			if (null != oriList && oriList.size() > 1 && !update.getId().equals(oriList.get(0).getId())) {
				throw new CustomException("编辑记录未找到");
			}
			if (!"流程未启动".equals(update.getStage())) {
				throw new CustomException("流程已启动不允许修改");
			}
			entity.setOrganizationId(String.join(",", organizationIds));//考评组织ID
			entity.setOrganizationName(String.join(",", organizationNames));
			entity.setOrganizationEvaluationRuleId(String.join(",", evaluationRuleIds));
			entity.setUpdateTime(new Date());
		}
		boolean b = super.saveOrUpdate(entity);
		return b;
	}


	@Override
	public List<DeptResponsibility> list(String organizationId, String checkCycle, String year, String month) {
		QueryWrapper<DeptResponsibility> qw = new QueryWrapper<>();
		if (StringUtils.isNotEmpty(organizationId)) {
			qw.lambda().eq(DeptResponsibility::getOrganizationId, organizationId);
		}

		if (StringUtils.isNotEmpty(checkCycle)) {
			qw.lambda().eq(DeptResponsibility::getCheckCycle, checkCycle);
		}
		if (StringUtils.isNotEmpty(year)) {
			qw.lambda().eq(DeptResponsibility::getYear, year);
		}
		if (StringUtils.isNotEmpty(month)) {
			qw.lambda().in(DeptResponsibility::getMonth, month);
		}
		List<DeptResponsibility> list = baseMapper.selectList(qw);
		return list;
	}

	@Transactional
	@Override
	public boolean removeByIds(Collection<? extends Serializable> idList) {

		QueryWrapper<ResponsibilityIndicatorInfo> infoqw = new QueryWrapper<>();
		infoqw.lambda().in(ResponsibilityIndicatorInfo::getDeptResponsibilityId, idList);
		int infcount = responsibilityIndicatorInfoService.count(infoqw);
		if (infcount > 0) {
			throw new CustomException("已发起流程不允许删除");
		}

		boolean b = super.removeByIds(idList);
		if (!b) {
			throw new CustomException("删除失败");

		}
		return true;
	}


}
