package com.ims.eval.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ims.common.utils.StringUtils;
import com.ims.eval.cache.CacheContext;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.EvaluateRule;
import com.ims.eval.dao.EvaluateRuleMapper;
import com.ims.eval.service.IEvaluateRuleService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * <p>
 * 考评规则表 服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-02-27
 */
@Service
public class EvaluateRuleServiceImpl extends ServiceImpl<EvaluateRuleMapper, EvaluateRule> implements IEvaluateRuleService {

	@Override
	public IPage<EvaluateRule> list(Integer pageNum, Integer pageSize, String id, String des, String binSection, String binStage) {

		QueryWrapper<EvaluateRule> qw = new QueryWrapper<>();
		if (null == pageNum || null == pageSize) {
			throw new RuntimeException("分页参数为空");
		}

		//构造分页构造器
		Page<EvaluateRule> page = new Page<>(pageNum, pageSize);

		if (StringUtils.isNotEmpty(id)) {
			qw.lambda().eq(EvaluateRule::getId, id);
		}

		if (StringUtils.isNotEmpty(des)) {
			qw.lambda().like(EvaluateRule::getDes, des);
		}

		if (StringUtils.isNotEmpty(binSection)) {
			qw.lambda().like(EvaluateRule::getBinSection, binSection);
		}

		if (StringUtils.isNotEmpty(binStage)) {
			qw.lambda().eq(EvaluateRule::getBinStage, binStage);
		}

		qw.lambda().orderByDesc(EvaluateRule::getCreateTime);

		IPage<EvaluateRule> list = baseMapper.selectPage(page, qw);
		list.getRecords().stream().forEach(l->{
			l.setBinSectionName(null == CacheContext.ddNameMap.get(l.getBinSection()) ? "" :CacheContext.ddNameMap.get(l.getBinSection()));
			l.setBinStageName(null == CacheContext.ddNameMap.get(l.getBinStage()) ? "" : CacheContext.ddNameMap.get(l.getBinStage()));
		});
		return list;
	}

	@Override
	public IPage<EvaluateRule> listPage(Integer pageNum, Integer pageSize, String id, String des, String binSection, String binStage) {
		if (null == pageNum || null == pageSize) {
			throw new RuntimeException("分页参数为空");
		}
		//构造分页构造器
		Page<EvaluateRule> page = new Page<>(pageNum, pageSize);
		IPage<EvaluateRule> list = baseMapper.selectListPage(page, id, des, binSection, binStage);
		return list;
	}

	@Override
	public List<EvaluateRule> listAll(String id, String des, String binSection, String binStage) {
		QueryWrapper<EvaluateRule> qw = new QueryWrapper<>();

		if (StringUtils.isNotEmpty(id)) {
			qw.lambda().eq(EvaluateRule::getId, id);
		}

		if (StringUtils.isNotEmpty(des)) {
			qw.lambda().like(EvaluateRule::getDes, des);
		}

		if (StringUtils.isNotEmpty(binSection)) {
			qw.lambda().like(EvaluateRule::getBinSection, binSection);
		}

		if (StringUtils.isNotEmpty(binStage)) {
			qw.lambda().eq(EvaluateRule::getBinStage, binStage);
		}

		qw.lambda().orderByDesc(EvaluateRule::getCreateTime);

		List<EvaluateRule> list = baseMapper.selectList(qw);

		list.stream().forEach(l->{
			l.setBinSectionName(null == CacheContext.ddNameMap.get(l.getBinSection()) ? "" :CacheContext.ddNameMap.get(l.getBinSection()));
			l.setBinStageName(null == CacheContext.ddNameMap.get(l.getBinStage()) ? "" : CacheContext.ddNameMap.get(l.getBinStage()));
		});

		return list;
	}

	@Override
	public List<EvaluateRule> listAll(List<String> ids) {
		QueryWrapper<EvaluateRule> qw = new QueryWrapper<>();

		if (null == ids || ids.size()<=0) {
			throw new CustomException("获取规则参数空");
		}
		qw.lambda().in(EvaluateRule::getId, ids);
		List<EvaluateRule> list = baseMapper.selectList(qw);
		return list;
	}

	@Override
	public boolean saveOrUpdate(EvaluateRule entity) {

		if (null != entity && (null == entity.getId() || "".equals(entity.getId().trim()))) {
			QueryWrapper<EvaluateRule> qw = new QueryWrapper<>();
			qw.lambda().eq(EvaluateRule::getBinSection, entity.getBinSection());
			qw.lambda().eq(EvaluateRule::getBinStage, entity.getBinStage());
			qw.lambda().eq(EvaluateRule::getEnable, true);
			List<EvaluateRule> list = baseMapper.selectList(qw);
			if (null != list && list.size() > 0) {
				throw new CustomException("当前模块规则已存在,如若新增规则，请禁用已存在规则");
			}

			entity.setCreateTime(new Date());//创建时间
			entity.setEnable(true);//默认启用
		} else {
			entity.setUpdateTime(new Date());
		}
		return super.saveOrUpdate(entity);

	}
}
