package com.ims.eval.service.impl;

import com.ims.common.utils.StringUtils;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.IndicatorDictionary;
import com.ims.eval.entity.Myuser;
import com.ims.eval.entity.OrganizationEvaluationInfo;
import com.ims.eval.dao.OrganizationEvaluationInfoMapper;
import com.ims.eval.entity.dto.response.OrganizationEvaluationInfoResDTO;
import com.ims.eval.service.ICalculateIndicatorItemInfoService;
import com.ims.eval.service.IIndicatorDictionaryService;
import com.ims.eval.service.IOrganizationEvaluationInfoService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ims.eval.service.IUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 考评指标明细

 服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-03-16
 */
@Service
public class OrganizationEvaluationInfoServiceImpl extends ServiceImpl<OrganizationEvaluationInfoMapper, OrganizationEvaluationInfo> implements IOrganizationEvaluationInfoService {

	@Autowired
	private ICalculateIndicatorItemInfoService iCalculateIndicatorItemInfoService;


	@Autowired
	private IIndicatorDictionaryService indicatorDictionaryService;

	@Autowired
	private IUserService userService;

	@Override
	public Map finishValueList(String organizationEvaluationId, String dept, HttpServletRequest request) {

		Myuser myuser = userService.getSysUser(request);
		if(null == myuser){
			throw new CustomException("暂无权限");
		}

		List<OrganizationEvaluationInfoResDTO> list = baseMapper.selectListEvaluationInfoId(organizationEvaluationId,dept,"");//默认获取指定的怎目标
		list.stream().forEach(i->{
			i.setUpdateMark(i.getDeptName().equals(myuser.getDeptName()));
		});
		Map<String  ,List<OrganizationEvaluationInfoResDTO>> map  = list.stream().collect(Collectors.groupingBy(OrganizationEvaluationInfoResDTO::getIsQuantified));
		map.put("quantifiedList", map.remove("是"));
		map.put("nonQuantifiedList", map.remove("否"));
		return map;
	}

	@Transactional
	@Override
	public boolean saveBatchDto(List<OrganizationEvaluationInfo> organizationEvaluationInfoList) throws Exception {
		for (OrganizationEvaluationInfo entity : organizationEvaluationInfoList) {

			if (null != entity && null != entity.getId() && !"".equals(entity.getId().trim())) {
				OrganizationEvaluationInfo info = baseMapper.selectById(entity.getId());
				info.setQuantifiedValue(entity.getQuantifiedValue());//量化值
				info.setNonQuantifiedValue(entity.getNonQuantifiedValue());//非量化值
				info.setUpdateTime(new Date());//更新时间
				info.setUpdateBy(entity.getUpdateBy());//更新者
				info.setRemark(entity.getRemark());//备注
				info.setState(entity.getState());//初始状态
				boolean b = super.saveOrUpdate(info);
				if(!b){
					throw new CustomException("更新失败");
				}
			}else {
				entity.setCreateTime(new Date());

				if (StringUtils.isEmpty(entity.getIndicatorId())) {
					throw new CustomException("保存失败");
				}
				List<IndicatorDictionary> dictionaryList =  indicatorDictionaryService.list("",entity.getIndicatorId(),"","");
				for(IndicatorDictionary d : dictionaryList){

					OrganizationEvaluationInfo addinfo = new OrganizationEvaluationInfo();
					addinfo.setOrganizationEvaluationId(entity.getOrganizationEvaluationId());
					addinfo.setOrganizationEvaluationRuleId(entity.getOrganizationEvaluationRuleId());
					addinfo.setIndicatorId(entity.getIndicatorId());
					addinfo.setIndicatorDictionaryId(d.getId());
					addinfo.setOptionCode(d.getOptionCode());
					addinfo.setIsQuantified(entity.getIsQuantified());
					addinfo.setQuantifiedValue(entity.getQuantifiedValue());
					addinfo.setNonQuantifiedValue(entity.getNonQuantifiedValue());
					addinfo.setState("-1");//初始状态
					addinfo.setCreateTime(new Date());

					boolean b = super.saveOrUpdate(addinfo);
					if(!b){
						throw new CustomException("保存失败");
					}
				}

			}

		}

//		OrganizationEvaluationInfo organizationEvaluationInfo = baseMapper.selectById(organizationEvaluationInfoList.get(0).getId());
		//查询本次考评所有考评指标明细
		List<OrganizationEvaluationInfo> organizationEvaluationInfos = baseMapper.selectByEvaluationInfoId(organizationEvaluationInfoList.get(0).getOrganizationEvaluationId());

		//调用需要计算指标项的方法
		boolean calculate = calculateIdicatorItem(organizationEvaluationInfos);

		//调用计算存储最大值，最小值的方法
		if (calculate){
			iCalculateIndicatorItemInfoService.saveIndicatorInfo(organizationEvaluationInfos);
		}


		return true;
	}


	//todo  后续添加需要计算的指标项
	public boolean calculateIdicatorItem(List<OrganizationEvaluationInfo> organizationEvaluationInfos){


		//根据考评规则分组
		Map<String, List<OrganizationEvaluationInfo>> collect1 = organizationEvaluationInfos.stream().collect(Collectors.groupingBy(OrganizationEvaluationInfo::getOrganizationEvaluationRuleId));
		for (Map.Entry<String, List<OrganizationEvaluationInfo>> entry1 : collect1.entrySet()){
			//根据指标id分组
			Map<String, List<OrganizationEvaluationInfo>> collect = entry1.getValue().stream().collect(Collectors.groupingBy(OrganizationEvaluationInfo::getIndicatorId));
			for(Map.Entry<String, List<OrganizationEvaluationInfo>> entry : collect.entrySet()){
				//过滤差值考评指标明细
				List<OrganizationEvaluationInfo> cz = entry.getValue().stream().filter(r -> r.getOptionCode().equals("CZ")).collect(Collectors.toList());
				if (cz.size() == 1){
					List<OrganizationEvaluationInfo> wcz = entry.getValue().stream().filter(r -> r.getOptionCode().equals("WCZ")).collect(Collectors.toList());
					List<OrganizationEvaluationInfo> value = entry.getValue().stream().filter(r -> r.getOptionCode().equals("JHZ") || r.getOptionCode().equals("DBZ") || r.getOptionCode().equals("MBZ")).collect(Collectors.toList());

					//完成值与目标值 ||计划值||对标值进行计算
					if (wcz.size() == 1 && value.size() == 1){
						double wczValue = wcz.get(0).getQuantifiedValue();
						double dbzvalue = value.get(0).getQuantifiedValue();
						cz.get(0).setQuantifiedValue(wczValue - dbzvalue);
						cz.get(0).setCreateTime(new Date());
						boolean b = super.saveOrUpdate(cz.get(0));
						if(!b){
							throw new CustomException("更新失败");
						}
					}
				}

				//过滤单位千瓦利润考评指标明细
				List<OrganizationEvaluationInfo> dwqwlr = entry.getValue().stream().filter(r -> r.getOptionCode().equals("DWQWLR")).collect(Collectors.toList());
				if (dwqwlr.size() == 1){
					List<OrganizationEvaluationInfo> lrgxl = entry.getValue().stream().filter(r -> r.getOptionCode().equals("LRGXL")).collect(Collectors.toList());
					List<OrganizationEvaluationInfo> zjrl = entry.getValue().stream().filter(r -> r.getOptionCode().equals("ZJRL")).collect(Collectors.toList());

					//完成值与目标值 ||计划值||对标值进行计算
					if (lrgxl.size() == 1 && zjrl.size() == 1){
						double wczValue = lrgxl.get(0).getQuantifiedValue();
						double dbzvalue = zjrl.get(0).getQuantifiedValue();
						if (dbzvalue == 0 || dbzvalue == 0.0){
							dwqwlr.get(0).setQuantifiedValue(0);
							dwqwlr.get(0).setCreateTime(new Date());
							boolean b = super.saveOrUpdate(dwqwlr.get(0));
							if(!b){
								throw new CustomException("更新失败");
							}
						}else{
							//double保留两位小数
							double v = new BigDecimal(wczValue / dbzvalue).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
							dwqwlr.get(0).setQuantifiedValue(v);
							dwqwlr.get(0).setCreateTime(new Date());
							boolean b = super.saveOrUpdate(dwqwlr.get(0));
							if(!b){
								throw new CustomException("更新失败");
							}
						}
					}
				}

			}
		}
		return true;
	}
}
