package com.ims.eval.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ims.common.utils.StringUtils;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.YearGroupCoefficient;
import com.ims.eval.entity.YearOperatingCoefficient;
import com.ims.eval.dao.YearOperatingCoefficientMapper;
import com.ims.eval.service.IYearOperatingCoefficientService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * <p>
 * 年度经营业绩系数 服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-03-17
 */
@Service
public class YearOperatingCoefficientServiceImpl extends ServiceImpl<YearOperatingCoefficientMapper, YearOperatingCoefficient> implements IYearOperatingCoefficientService {

	@Override
	public IPage<YearOperatingCoefficient> list(Integer pageNum, Integer pageSize, String id, String organizationName, String organizationId, String yearGroupCoefficientId, String binSection, String year) {
		QueryWrapper<YearOperatingCoefficient> qw = new QueryWrapper<>();
		if (null == pageNum || null == pageSize) {
			throw new CustomException("分页参数为空");
		}
		//构造分页构造器
		Page<YearOperatingCoefficient> page = new Page<>(pageNum, pageSize);

		if (StringUtils.isNotEmpty(id)) {
			qw.lambda().eq(YearOperatingCoefficient::getId, id);
		}

		if (StringUtils.isNotEmpty(organizationName)) {
			qw.lambda().like(YearOperatingCoefficient::getOrganizationName, organizationName);
		}
		if (StringUtils.isNotEmpty(organizationId)) {
			qw.lambda().eq(YearOperatingCoefficient::getOrganizationId, organizationId);
		}
		if (StringUtils.isNotEmpty(yearGroupCoefficientId)) {
			qw.lambda().eq(YearOperatingCoefficient::getYearGroupCoefficientId, yearGroupCoefficientId);
		}
		if (StringUtils.isNotEmpty(binSection)) {
			qw.lambda().eq(YearOperatingCoefficient::getBinSection, binSection);
		}

		if (StringUtils.isNotEmpty(year)) {
			qw.lambda().eq(YearOperatingCoefficient::getYear, year);
		}

		IPage<YearOperatingCoefficient> list = baseMapper.selectPage(page, qw);

		return list;
	}

	@Override
	public List<YearOperatingCoefficient> listAll(String id, String organizationName, String organizationId, String yearGroupCoefficientId, String binSection, String year) {

		QueryWrapper<YearOperatingCoefficient> qw = new QueryWrapper<>();

		if (StringUtils.isNotEmpty(id)) {
			qw.lambda().eq(YearOperatingCoefficient::getId, id);
		}

		if (StringUtils.isNotEmpty(organizationName)) {
			qw.lambda().like(YearOperatingCoefficient::getOrganizationName, organizationName);
		}
		if (StringUtils.isNotEmpty(organizationId)) {
			qw.lambda().eq(YearOperatingCoefficient::getOrganizationId, organizationId);
		}
		if (StringUtils.isNotEmpty(yearGroupCoefficientId)) {
			qw.lambda().eq(YearOperatingCoefficient::getYearGroupCoefficientId, yearGroupCoefficientId);
		}
		if (StringUtils.isNotEmpty(binSection)) {
			qw.lambda().eq(YearOperatingCoefficient::getBinSection, binSection);
		}

		if (StringUtils.isNotEmpty(year)) {
			qw.lambda().eq(YearOperatingCoefficient::getYear, year);
		}
		List<YearOperatingCoefficient> list = baseMapper.selectList( qw);

		return list;
	}
}
